function[Output]=boot(k,In)
%       function[Output]=boot(k,In)
%    
%   Bootstrap  resampling  procedure  for  univariate  (In) 
%   input data. For an input vector the  program  generates
%   a bootstrap vector (of the size k), which elements  are
%   the samples with replacement  from  the  input  vector.
%   For an input  matrix  it generates a  bootstrap  matrix 
%   (of the same numbers of columns as the  input  one  but 
%   with k rows), which elements in columns are the samples 
%   with  replacement from the columns of  the input matrix.
% 

%
%   Modified by D. Reid and D.R. Iskander 15/12/94
%

%rand('uniform')    (for Octave)

[s1,s2]=size(In);     
  if s1==1,  
    Output=In(fix(s2*rand(1,k)+1)); 
  elseif s2==1  
    Output=In(fix(s1*rand(k,1)+1)); 
  else
         ind=fix(s1*rand(k,s2)+1)+ones(1,k)'*[0:s1:(s1*s2)-1];
         Output=In(ind);
  end;

% References
% 
% Efron, B.  Bootstrap Methods. Another Look at the Jackknife. 
%            The Annals of Statistics, Vol. 7, pp. 1-26, 1979.
%
% Zoubir, A.M. Bootstrap: Theory and Applications. Proceedings 
%              of the SPIE 1993 Conference on Advanced  Signal 
%              Processing Algorithms, Architectures and Imple-
%              mentations. pp. 216-235, San Diego, July  1993.
