function ciestpb()
%Bootstrap routine for confidence interval estimation (parametric bootstrap)

%Get handles
stath=findobj(gcbf,'Tag','PopupMenu1');
disth=findobj(gcbf,'Tag','PopupMenu2');
pdisth=findobj(gcbf,'Tag','PopupMenu3');
nh=findobj(gcbf,'Tag','EditText_N');
if ~chkno(nh,'The number of samples')
  return;
end
Bh=findobj(gcbf,'Tag','EditText_B1');
if ~chkno(Bh,'B1')
  return;
end
MCh=findobj(gcbf,'Tag','EditText_MC');
if ~chkno(MCh,'The number of Monte Carlo runs')
  return;
end
alphah=findobj(gcbf,'Tag','EditText_alpha');
if ~chkalpha(alphah)
  return;
end
outputh=findobj(gcbf,'Tag','Axes1');
waitbarh=findobj(gcbf,'Tag','Waitbar');
applybtnh=findobj(gcbf,'Tag','Pushbutton1');

%Get Matlab expressions from popup menus
matlabexp=get(stath,'UserData');
statfun=matlabexp{get(stath,'Value')};
matlabexp=get(disth,'UserData');
distfun=matlabexp{get(disth,'Value')};
matlabexp=get(pdisth,'UserData');
pdistfun=matlabexp{get(pdisth,'Value')};

%Get settings from edit boxes
B=str2num(get(Bh,'String'));
n=str2num(get(nh,'String'));
MC=str2num(get(MCh,'String'));
alpha=str2num(get(alphah,'String'));

%Change Apply button to Cancel button
applycb=get(applybtnh,'Callback');
set(applybtnh, 'String','Cancel', ...
  'Callback',['set(findobj(gcbf,''Tag'',''Pushbutton1''),''String'',''Apply'',''Callback'',''' applycb ''');']);

%Run bootstrap procedure
q1=floor(B*alpha/2);
if q1<1
  q1=1;
  B=ceil(2/alpha);
  h=warndlg(sprintf('Minimum number of bootstrap replications required for the significance level chosen is %d.',B));
  set(h,'WindowStyle','modal');
  uiwait(h);
  set(Bh,'String',num2str(B));
end
q2=B-q1+1;
x=eval(distfun); %e.g. distfun='randn(n,MC)'
lower=zeros(1,MC);
upper=zeros(1,MC);
i=1;
while i<=MC & strcmp(get(applybtnh,'String'),'Cancel')
  X=x(:,i,:);
  X=eval(pdistfun); %e.g. pdistfun='std(X)*randn(n,B)+mean(X)'
  bootstat=sort(eval(statfun)); %e.g. statfun='mean(X)'
  lower(i)=bootstat(q1);
  upper(i)=bootstat(q2);
  set(waitbarh,'XData',[0 i/MC i/MC 0]);drawnow;
  i=i+1;
end
set(waitbarh,'XData',[0 0 0 0]);

if i<=MC %Cancel button was pressed
  return; %do not output results
end

%Output results
fontsize=get(outputh,'FontSize');
fontweight=get(outputh,'FontWeight');
axes(outputh); cla; grid off;
if MC==1
  hist(bootstat,20);
  title(sprintf('Bootstrap Histogram (C.I.= [%.2f,%.2f])',lower,upper));
  xlabel('Bootstrap statistics');
  ylabel('Frequency of occurrence');
else
  plot(1:MC,[lower;upper]);
  grid on;
  title(sprintf('Bootstrap Confidence Intervals (Average C.I.= [%.2f,%.2f])',mean(lower),mean(upper)));
  xlabel('Monte Carlo run');
  ylabel('Parameter value');
end
%Ensure crucial properties are not changed after plotting
set(outputh,'Tag','Axes1','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'title'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'xlabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'ylabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);

%Restore Apply button
set(applybtnh,'String','Apply','Callback',applycb);
