function cipttut()
%Bootstrap tutorial routine for confidence interval estimation (Percentile-t)

%Get handles
stath=findobj(gcbf,'Tag','PopupMenu1');
disth=findobj(gcbf,'Tag','PopupMenu3');
nh=findobj(gcbf,'Tag','EditText_N');
if ~chkno(nh,'The number of samples')
  return;
end
Bh=findobj(gcbf,'Tag','EditText_B1');
if ~chkno(Bh,'B1')
  return;
end
B2h=findobj(gcbf,'Tag','EditText_B2');
if ~chkno(B2h,'B2')
  return;
end
MCh=findobj(gcbf,'Tag','EditText_MC');
if ~chkno(MCh,'The number of Monte Carlo runs')
  return;
end
alphah=findobj(gcbf,'Tag','EditText_alpha');
if ~chkalpha(alphah)
  return;
end
outputh=findobj(gcbf,'Tag','Axes1');
waitbarh=findobj(gcbf,'Tag','Waitbar');
startbtnh=findobj(gcbf,'Tag','Pushbutton1');

%Get Matlab expressions from popup menus
matlabexp=get(stath,'UserData');
statfun=matlabexp{get(stath,'Value')};
matlabexp=get(disth,'UserData');
distfun=matlabexp{get(disth,'Value')};

%Get settings from edit boxes
B=str2num(get(Bh,'String'));
B2=str2num(get(B2h,'String'));
n=str2num(get(nh,'String'));
MC=str2num(get(MCh,'String'));
alpha=str2num(get(alphah,'String'));

%Change Start button to Continue button
startcb=get(startbtnh,'Callback');
set(startbtnh, 'String','Continue', 'Callback','uiresume(gcbf)');

%Run bootstrap procedure

%Output results as we go
fontsize=get(outputh,'FontSize');
fontweight=get(outputh,'FontWeight');
axes(outputh); cla; title(''); axis off;

q1=floor(B*alpha/2);
if q1<1
  q1=1;
  B=ceil(2/alpha);
  h=warndlg(sprintf('Minimum number of bootstrap replications required for the significance level chosen is %d.',B));
  set(h,'WindowStyle','modal');
  uiwait(h);
  set(Bh,'String',num2str(B));
end
q2=B-q1+1;

axes(outputh);
text(0.05,0.95,'Step 0. Experiment: Collect data.','Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
x=eval(distfun); %e.g. distfun='randn(n,MC)'
X=x;
if n>10
  s=['X = (' sprintf('%.2f,',X(1:9)) '...)'];
else
  s=['X = (' sprintf('%.2f,',X(1:end-1)) sprintf('%.2f',X(end)) ')'];
end
text(0.1,0.9,s,'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.15 0.15 0]);
uiwait(gcbf);

axes(outputh);
text(0.05,0.8,'Step 1. Parameter Estimation: Estimate parameter and std. error.','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
vhat=eval(statfun);
X=X(ceil(n*rand(n,B2)));
sigma1=std(eval(statfun));
text(0.1,0.75,sprintf('{\\it P} = %g',vhat),'Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
text(0.1,0.7,sprintf('{\\it s} = %g',sigma1),'Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.3 0.3 0]);
uiwait(gcbf);

axes(outputh);
text(0.05,0.6,'Step 2. Resampling: Draw resample with replacement from data.','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
X=x(ceil(n*rand(n,B)));
if n>10
  s=['X* = (' sprintf('%.2f,',X(1:9)) '...)'];
else
  s=['X* = (' sprintf('%.2f,',X(1:n-1)) sprintf('%.2f',X(n)) ')'];
end
text(0.1,0.55,s,'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.45 0.45 0]);
uiwait(gcbf);

axes(outputh);
text(0.05,0.45,'Step 3. Calculation of the pivotal statistic: Form ({\it P}*-{\it P})/{\it s}*.','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
vhatstar=eval(statfun);
Xs=X;
sigma=zeros(1,B);
for b=1:B
  X=Xs(:,b);
  X=X(ceil(n*rand(n,B2)));
  sigma(b)=std(eval(statfun));
end
bootstat=(vhatstar-vhat)./sigma;       
text(0.1,0.4,sprintf('{\\it T}* = %g',bootstat(1)),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.6 0.6 0]);
uiwait(gcbf);

axes(outputh);
text(0.05,0.3,'Step 4. Repetition: Repeat Steps 1 and 2 for B1 bootstrap estimates.','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
if B>10
  s=['T* = (' sprintf('%.2f,',bootstat(1:9)) '...)'];
else
  s=['T* = (' sprintf('%.2f,',bootstat(1:end-1)) sprintf('%.2f',bootstat(end)) ')'];
end
text(0.1,0.25,s,'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.75 0.75 0]);
uiwait(gcbf);

axes(outputh);
text(0.05,0.15,'Step 5. Approximation of Distribution: Sort bootstrap estimates.','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
bootstat=sort(bootstat);
if B>10
  s=['T* = (' sprintf('%.2f,',bootstat(1:8)) sprintf('...,%.2f)',bootstat(end))];
else
  s=['T* = (' sprintf('%.2f,',bootstat(1:end-1)) sprintf('%.2f',bootstat(end)) ')'];
end
text(0.1,0.1,s,'Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.9 0.9 0]);
uiwait(gcbf);

axes(outputh); cla;
text(0.05,0.95,'Step 6. Confidence Interval: The 100(1-\alpha)% c. i. is','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
text(0.05,0.87,'[{\it P}-{\it s}{\it T}*(q_2),{\it P}-{\it s}{\it T}*(q_1)].','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
text(0.1,0.8,sprintf('q_1 = [B1\\alpha/2] = %d',q1),'Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
text(0.1,0.75,sprintf('q_2 = B1-q_1+1 = %d',q2),'Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
lower=vhat-bootstat(q1)*sigma1;
upper=vhat-bootstat(q2)*sigma1;
text(0.1,0.7,sprintf('C.I. = [%.2f,%.2f]',lower,upper),'Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 1 1 0]);
set(startbtnh, 'String','End');
uiwait(gcbf);

set(waitbarh,'XData',[0 0 0 0]);

%Output results
axes(outputh);
hist(bootstat,20);
title(sprintf('Bootstrap Histogram (C.I.= [%.2f,%.2f])',lower,upper));
xlabel('Bootstrap statistics');
ylabel('Frequency of occurrence');

%Ensure crucial properties are not changed after plotting
set(outputh,'Tag','Axes1','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'title'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'xlabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'ylabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);

%Restore Start button
set(startbtnh,'String','Start','Callback',startcb);
