function civstut()
%Bootstrap tutorial routine for confidence interval estimation (Variance Stabilisation)

%Get handles
stath=findobj(gcbf,'Tag','PopupMenu1');
disth=findobj(gcbf,'Tag','PopupMenu3');
nh=findobj(gcbf,'Tag','EditText_N');
if ~chkno(nh,'The number of samples')
  return;
end
B1h=findobj(gcbf,'Tag','EditText_B1');
if ~chkno(B1h,'B1')
  return;
end
B2h=findobj(gcbf,'Tag','EditText_B2');
if ~chkno(B2h,'B2')
  return;
end
B3h=findobj(gcbf,'Tag','EditText_B3');
if ~chkno(B3h,'B3')
  return;
end
MCh=findobj(gcbf,'Tag','EditText_MC');
if ~chkno(MCh,'The number of Monte Carlo runs')
  return;
end
alphah=findobj(gcbf,'Tag','EditText_alpha');
if ~chkalpha(alphah)
  return;
end
outputh=findobj(gcbf,'Tag','Axes1');
waitbarh=findobj(gcbf,'Tag','Waitbar');
startbtnh=findobj(gcbf,'Tag','Pushbutton1');

%Get Matlab expressions from popup menus
matlabexp=get(stath,'UserData');
statfun=matlabexp{get(stath,'Value')};
matlabexp=get(disth,'UserData');
distfun=matlabexp{get(disth,'Value')};

%Get settings from edit boxes
B1=str2num(get(B1h,'String'));
B2=str2num(get(B2h,'String'));
B3=str2num(get(B3h,'String'));
n=str2num(get(nh,'String'));
MC=str2num(get(MCh,'String'));
alpha=str2num(get(alphah,'String'));

%Change Start button to Continue button
startcb=get(startbtnh,'Callback');
set(startbtnh, 'String','Continue', 'Callback','uiresume(gcbf)');

%Run bootstrap procedure

%Output results as we go
fontsize=get(outputh,'FontSize');
fontweight=get(outputh,'FontWeight');
axes(outputh); cla; title(''); axis off;

q1=floor(B3*alpha/2);
if q1<1
  q1=1;
  B3=ceil(2/alpha);
  h=warndlg(sprintf('Minimum number of bootstrap replications required for the significance level chosen is %d.',B3));
  set(h,'WindowStyle','modal');
  uiwait(h);
  set(B3h,'String',num2str(B3));
end
q2=B3-q1+1;

axes(outputh);
text(0.05,0.95,'Step 0. Experiment: Collect data.','Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
x=eval(distfun); %e.g. distfun='randn(n,MC)'
X=x;
S=eval(statfun);
if n>10
  str=['X = (' sprintf('%.2f,',X(1:9)) '...)'];
else
  str=['X = (' sprintf('%.2f,',X(1:end-1)) sprintf('%.2f',X(end)) ')'];
end
text(0.1,0.9,str,'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.15 0.15 0]);
uiwait(gcbf);

axes(outputh);
text(0.05,0.8,'Step 1. Estimation of the variance stabilising transformation.','Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);

axes(outputh);
text(0.05,0.7,'(a) Generate B1 bootstrap samples and for each calculate statistic.','Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
Xstar=x(ceil(n*rand(n,B1)));
X=Xstar;
Sstar=eval(statfun);
if B1>10
  str=['P* = (' sprintf('%.2f,',Sstar(1:9)) '...)'];
else
  str=['P* = (' sprintf('%.2f,',Sstar(1:end-1)) sprintf('%.2f',Sstar(end)) ')'];
end
text(0.1,0.65,str,'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.3 0.3 0]);
uiwait(gcbf);

axes(outputh);
text(0.05,0.55,'(b) Generate B2 bootstrap samples for each and estimate variance.','Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
vstar=zeros(1,B1);
Xstar2=zeros(n,B2);
for b=1:B1
  Xstar2(:)=Xstar(ceil(n*rand(n,B2)),b);
  X=Xstar2;
  vstar(b)=var(eval(statfun));
end
if B1>10
  str=['s*^2 = (' sprintf('%.2f,',vstar(1:9)) '...)'];
else
  str=['s*^2 = (' sprintf('%.2f,',vstar(1:end-1)) sprintf('%.2f',vstar(end)) ')'];
end
text(0.1,0.5,str,'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
text(0.05,0.4,'Press Continue to view {\it s}* versus {\it P}* ...','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.45 0.45 0]);
uiwait(gcbf);

axes(outputh); cla;
plot(Sstar,sqrt(vstar),'*');
xlabel('Bootstrap statistics');
ylabel('Standard deviation');
%Ensure crucial properties are not changed after plotting
set(outputh,'Tag','Axes1','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'title'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'xlabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'ylabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
uiwait(gcbf);

axes(outputh); cla; axis off;
text(0.05,0.95,'(c) Estimate the variance function by smoothing {\it s}*^2 against {\it P}*.','Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
[s,vstar,v]=smooth(Sstar,log(vstar),0.5);
vstar=exp(vstar);
v=exp(v);
text(0.05,0.85,'Press Continue to view smoothed {\it s}* versus {\it P}* ...','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.6 0.6 0]);
uiwait(gcbf);

axes(outputh); cla;
plot(s,sqrt(vstar),'*',s,sqrt(v),'-');
xlabel('Bootstrap statistics');
ylabel('Standard deviation');
%Ensure crucial properties are not changed after plotting
set(outputh,'Tag','Axes1','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'title'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'xlabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'ylabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
uiwait(gcbf);

axes(outputh); cla; axis off;
text(0.05,0.95,'(d) Estimate the variance stabilising transformation {\it h}({\it P}).','Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
f=v.^(-0.5);
h=cumsum([0 0.5*(f(1:end-1)+f(2:end)).*diff(s)]);
text(0.05,0.85,'Press Continue to view {\it h}({\it p}) versus {\it p} ...','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.75 0.75 0]);
uiwait(gcbf);

axes(outputh); cla;
plot(s,h);
xlabel('Parameter');
ylabel('Variance stabilising transformation');
%Ensure crucial properties are not changed after plotting
set(outputh,'Tag','Axes1','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'title'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'xlabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'ylabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
uiwait(gcbf);

axes(outputh); cla; axis off;
text(0.05,0.95,'Step 2. Quantile Estimation: Generate B3 bootstrap samples and','Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
text(0.05,0.9,'compute {\it P}* and {\it h}({\it P}*) for each.  Approximate the distribution','Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
text(0.05,0.85,'of {\it h}({\it P})-{\it h}({\it p}) by that of {\it T}* = {\it h}({\it P}*)-{\it h}({\it P}).','Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
X=x(ceil(n*rand(n,B3)));
Sstar=eval(statfun);
hS=linterp(s,h,S);
hSstar=linterp(s,h,Sstar);
y=sort(hSstar-hS);
if B3>10
  str=['P* = (' sprintf('%.2f,',Sstar(1:9)) '...)'];
else
  str=['P* = (' sprintf('%.2f,',Sstar(1:end-1)) sprintf('%.2f',Sstar(end)) ')'];
end
text(0.1,0.8,str,'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
if B3>10
  str=['{\it h}(P*) = (' sprintf('%.2f,',hSstar(1:9)) '...)'];
else
  str=['{\it h}(P*) = (' sprintf('%.2f,',hSstar(1:end-1)) sprintf('%.2f',hS(end)) ')'];
end
text(0.1,0.75,str,'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
if B3>10
  str=['T* = (' sprintf('%.2f,',y(1:8)) sprintf('...,%.2f)',y(end))];
else
  str=['T* = (' sprintf('%.2f,',y(1:end-1)) sprintf('%.2f',y(end)) ')'];
end
text(0.1,0.7,str,'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.9 0.9 0]);
uiwait(gcbf);

axes(outputh);
text(0.05,0.6,'Step 3. Confidence Interval: The 100(1-\alpha)% c. i. is','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
text(0.05,0.52,'[{\it h}^{-1}({\it h}({\it P})-{\it T}*(q_2)),{\it h}^{-1}({\it h}({\it P})-{\it T}*(q_2))].','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
text(0.1,0.45,sprintf('q_1 = [B1\\alpha/2] = %d',q1),'Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
text(0.1,0.4,sprintf('q_2 = B1-q_1+1 = %d',q2),'Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
lower=linterp(h,s,hS-y(q2));
upper=linterp(h,s,hS-y(q1));
text(0.1,0.35,sprintf('C.I. = [%.2f,%.2f]',lower,upper),'Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 1 1 0]);
set(startbtnh, 'String','End');
uiwait(gcbf);

set(waitbarh,'XData',[0 0 0 0]);

%Output results
axes(outputh);
hist(y,20);
title(sprintf('Bootstrap Histogram (C.I.= [%.2f,%.2f])',lower,upper));
xlabel('Bootstrap statistics');
ylabel('Frequency of occurrence');

%Ensure crucial properties are not changed after plotting
set(outputh,'Tag','Axes1','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'title'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'xlabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'ylabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);

%Restore Start button
set(startbtnh,'String','Start','Callback',startcb);


function [x_sort, y_sort, y_sm] = smooth(x, y, w)
%          [x_sort, y_sort, y_sm] = smooth(x, y, w);
%          A running line smoother that fits the data by linear 
%          least squares. Used to compute the variance stabilising 
%          transformation.
%
%    Inputs:  
%          x - one or more columns of covariates
%          y - one column of response for each column of covariate
%          w - span, proportion of data in symmetric centred window
%              
%   Outputs: 
%     x_sort - sorted columns of x
%     y_sort - values of y associated with x_sort
%       y_sm - smoothed version of y
%
%  Note: If inputs are row vectors, operation is carried out row-wise.

%  Created by  Hwa-Tung Ong, 1996
%
% References
%
% Hastie, T.J. and Tibshirani, R.J. Generalised additive models.
%                Chapman and Hall, 1990.  
%
% Zoubir, A.M.   Bootstrap: Theory and Applications. Proceedings 
%                of the SPIE 1993 Conference on Advanced  Signal 
%                Processing Algorithms, Architectures and Imple-
%                mentations. pp. 216-235, San Diego, July  1993.
%
%  Zoubir, A.M. and Boashash, B. The Bootstrap: Signal Processing 
%               Applications. IEEE Transactions  on  Signal  Pro-
%               cessing, Vol. 15, No. 1, pp. 55-76, 1998.

if any(size(x) ~= size(y))
    error('Input matrices must be the same length.'),
end

[nr,nc] = size(x);
n=nr;
if (nr==1) x=x';y=y';n=nc;nc=1; end
y_sm = zeros(n,nc);
[x_sort,order] = sort(x);
for i = 1:nc y_sort(:,i) = y(order(:,i),i); end
k = fix(w*n/2);

for i = 1:n
    window = max(i-k,1):min(i+k,n);
    xwin = x_sort(window,:);
    ywin = y_sort(window,:);
    xbar = mean(xwin);
    ybar = mean(ywin);
    copy = ones(length(window),1);
    x_mc = xwin - copy*xbar;    % mc = mean-corrected
    y_mc = ywin - copy*ybar;
    y_sm(i,:) = sum(x_mc.*y_mc)./sum(x_mc.*x_mc) .* (x_sort(i,:)-xbar) + ybar;
end

if (nr==1) x_sort=x_sort';y_sort=y_sort';y_sm=y_sm'; end


function y0 = linterp(x, y, x0)
%linterp   Linear interpolation
%
%Description:
%          A Matlab function to map given abscissa values
%          to ordinates, performing linear interpolation
%          and extrapolation when the input table of data
%          does not provide the right coordinates.
%
%Usage:    y0 = linterp(x, y, x0);
%
%Inputs:   x       one or more sorted columns of abscissa values
%          y       corresponding columns of ordinates
%          x0      abscissa values to map in corresponding columns
%
%Outputs:  y0      interpolated values corresponding to x0
%
%Note:     If inputs are row vectors, operation is carried out row-wise.
%
%Author:   Hwa-Tung Ong, 1996

% check inputs
if any(size(x) ~= size(y))
    error('Input matrices X and Y must be the same size.')
end
[nr,nc] = size(x);
n=nr;
if (nr==1) x=x';y=y';x0=x0';n=nc;nc=1; end
np = size(x0,1);
if nc ~= size(x0,2)
    error('Input matrices must have the same number of columns.')
end

% map x0 to y0
y0 = zeros(np,nc);
for c = 1:nc

    % find index of closest x on the left side of x0
    k = sum(x(:,c)*ones(1,np) < ones(n,1)*x0(:,c)');

    % adjust index for x0 outside x range
    i = find(k == 0);
    k(i) = ones(size(i));
    i = find(k == n);
    k(i) = ones(size(i))*(n-1);

    % linear interpolate or extrapolate to get y0
    y0(:,c) = (y(k+1,c)-y(k,c))./(x(k+1,c)-x(k,c)) .* (x0(:,c)-x(k,c)) + y(k,c);
end

if (nr==1) y0=y0'; end
