function dmf()
%Bootstrap routine for matched filter detection

%Get handles
signalh=findobj(gcbf,'Tag','PopupMenu1');
noiseh=findobj(gcbf,'Tag','PopupMenu2');
sceneh=findobj(gcbf,'Tag','PopupMenu3');
nh=findobj(gcbf,'Tag','EditText_N');
if ~chkno(nh,'The number of samples')
  return;
end
Bh=findobj(gcbf,'Tag','EditText_B1');
if ~chkno(Bh,'B1')
  return;
end
MCh=findobj(gcbf,'Tag','EditText_MC');
if ~chkno(MCh,'The number of Monte Carlo runs')
  return;
end
alphah=findobj(gcbf,'Tag','EditText_alpha');
if ~chkalpha(alphah)
  return;
end
outputh=findobj(gcbf,'Tag','Axes1');
waitbarh=findobj(gcbf,'Tag','Waitbar');
applybtnh=findobj(gcbf,'Tag','Pushbutton1');

%Get Matlab expressions from popup menus
matlabexp=get(signalh,'UserData');
signalfun=matlabexp{get(signalh,'Value')};
matlabexp=get(noiseh,'UserData');
choice=get(noiseh,'Value');
noisefun=matlabexp{choice};
if choice==length(get(noiseh,'String')) %use the user-provided variance
  v=matlabexp{end}; %noise variance
else
  v=1;
end

%Get settings from edit boxes
B=str2num(get(Bh,'String'));
n=str2num(get(nh,'String'));
MC=str2num(get(MCh,'String'));
alpha=str2num(get(alphah,'String'));

%Change Apply button to Cancel button
applycb=get(applybtnh,'Callback');
set(applybtnh, 'String','Cancel', ...
  'Callback',['set(findobj(gcbf,''Tag'',''Pushbutton1''),''String'',''Apply'',''Callback'',''' applycb ''');']);

%Run bootstrap procedure
q=floor((B+1)*(1-alpha));
s=eval(signalfun);
w=eval(noisefun);
if get(sceneh,'Value')==1
  SNR=num2str(round(10*log10((s'*s/n)/v)));
  w=s(:,ones(1,MC))+w;
else
  SNR='-\infty';
end
lambda=zeros(1,MC);
T=zeros(1,MC);
i=1;
while i<=MC & strcmp(get(applybtnh,'String'),'Cancel')
  x=w(:,i);
  %Step 1
  theta_hat=s'*x/(s'*s);
  %Step 2
  w_hat=x-theta_hat*s;
  w_hat=w_hat-mean(w_hat);
  %Step 3
  x_ast=w_hat(ceil(n*rand(n,B)));
  %Step 4
  T_ast=s'*x_ast/(std(x,1)*sqrt(s'*s));
  %Step 5 Repeat Steps 3-4...
  %Step 6
  T_ast=sort(T_ast);
  lambda(i)=T_ast(q);
  %Step 7
  T(i)=s'*x/(sqrt(v*s'*s));
  set(waitbarh,'XData',[0 i/MC i/MC 0]);drawnow;
  i=i+1;
end
%Step 8
Pd=mean(T>lambda);
set(waitbarh,'XData',[0 0 0 0]);

if i<=MC %Cancel button was pressed
  return; %do not output results
end

%Output results
fontsize=get(outputh,'FontSize');
fontweight=get(outputh,'FontWeight');
axes(outputh); cla; grid off;
if MC==1
  hist(T_ast,20);
  hold on;
  a=axis;
  plot([lambda lambda],a(3:4),'r');
  plot([T T],a(3:4),'g');
  hold off;
  title(['Bootstrap Histogram (red: threshold, green: test statistic, SNR= ' SNR ' dB)']);
  xlabel('Bootstrap statistics');
  ylabel('Frequency of occurrence');
else
  plot(1:MC,lambda,'r',1:MC,T,'g');
  grid on;
  title(sprintf('red: threshold, green: test statistic (%%alarms= %d%%, SNR= %s dB)',round(100*Pd),SNR));
  xlabel('Monte Carlo run');
  ylabel('Bootstrap statistic value');
end
%Ensure crucial properties are not changed after plotting
set(outputh,'Tag','Axes1','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'title'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'xlabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'ylabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);

%Restore Apply button
set(applybtnh,'String','Apply','Callback',applycb);
