function modelsel()
%Bootstrap routine for confidence interval estimation (Percentile-t)

%Get handles
outputh=findobj(gcbf,'Tag','Axes1');
modelh=findobj(gcbf,'Tag','PopupMenu1');
disth=findobj(gcbf,'Tag','PopupMenu3');
nh=findobj(gcbf,'Tag','EditText_N');
if ~chkno(nh,'The number of samples')
  return;
end
Bh=findobj(gcbf,'Tag','EditText_B1');
if ~chkno(Bh,'B')
  return;
end
MCh=findobj(gcbf,'Tag','EditText_MC');
if ~chkno(MCh,'The number of Monte Carlo runs')
  return;
end

%orderh=findobj(gcbf,'Tag','EditText_alpha');
%
%if ~chkno(orderh,'Order')
%  return;
%end

outputh=findobj(gcbf,'Tag','Axes1');
waitbarh=findobj(gcbf,'Tag','Waitbar');
applybtnh=findobj(gcbf,'Tag','Pushbutton1');

%Get Matlab expressions from popup menus
matlabexp=get(modelh,'UserData');
model=matlabexp{get(modelh,'Value')};
matlabexp=get(disth,'UserData');
distfun=matlabexp{get(disth,'Value')};
%order=str2num(get(orderh,'String'));
%Get settings from edit boxes
B=str2num(get(Bh,'String'));
n=str2num(get(nh,'String'));
MC=str2num(get(MCh,'String'));
P=3;
%Change Apply button to Cancel button
applycb=get(applybtnh,'Callback');
set(applybtnh, 'String','Cancel', ...
  'Callback',['set(findobj(gcbf,''Tag'',''Pushbutton1''),''String'',''Apply'',''Callback'',''' applycb ''');']);

%Run bootstrap procedure

fontsize=get(outputh,'FontSize');
axes(outputh); cla; title(''); axis off;

t=(0:n-1)'; M=6;
signal=eval(model);
x=eval(distfun); %e.g. distfun='randn(n,MC)'
IND=zeros(ceil((2^(P+1)-1)/2),3);
i=1;
while i<=MC & strcmp(get(applybtnh,'String'),'Cancel')
   X = signal+x(:,i);
   [G,aic,mdl,models] = polymodel_ic99(X,B,P,M);                   
   [Min,ind]=min(G);
   IND(ind,1)=IND(ind,1)+1;
   [Min,ind]=min(aic);
   IND(ind,2)=IND(ind,2)+1;
   [Min,ind]=min(mdl);
   IND(ind,3)=IND(ind,3)+1;
   set(waitbarh,'XData',[0 i/MC i/MC 0]);drawnow;
   i=i+1;
end
set(waitbarh,'XData',[0 0 0 0]);

if i<=MC %Cancel button was pressed
  return; %do not output results
end

%Output results

fontsize=get(outputh,'FontSize');
fontweight=get(outputh,'FontWeight');
axes(outputh);
text(0.15,0.5,'Model Selection Results Displayed in a Separate Figure','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
figure
subplot(211);pltmat_color(models','Models','k','k',fontsize,fontweight)
ylabel('a_3     a_2     a_1     a_0','FontWeight',fontweight)
subplot(212);pltmat_color(IND','Model Selection Results','k','k',fontsize,fontweight)
ylabel('AIC        MDL        Boot','FontWeight',fontweight)
%Ensure crucial properties are not changed after plotting
set(outputh,'Tag','Axes1','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'title'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'xlabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'ylabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);

%Restore Apply button
set(applybtnh,'String','Apply','Callback',applycb);

%%%%%%%%%%%%%%%% Subfunctions %%%%%%%%%%%%%%%%%%%%
function pltmat_color(a,matrixName,gridColor,fontColor,fontSize,fontWeight);
%PLTMAT Display a matrix in a figure window.
%   PLTMAT(X,matrixName,gridColor,fontColor,fontSize,fontWeight)
%   takes the matrix X and displays it using gridColor for the
%   XColor and YColor, fontSize for the FontSize of each
%   matrix element, fontWeight for the FontWeight of each
%   matrix element and matrixName as the title for the plot. 

%   Ned Gulley, Denise Chen; June 1993.
%   Copyright (c) 1984-98 by The MathWorks, Inc.
%   $Revision: 5.5 $  $Date: 1997/11/21 23:26:47 $

% Now this function allows to change the font color as well.
% D. R. Iskander March 1999

% Clear the current axis
cla reset;
[m,n]=size(a);

% Place the text in the correct locations
% Index over number of rows
    for rowCnt=1:m,
        % Index over number of columns
        for colCnt=1:n,
            numberString=num2str(a(rowCnt,colCnt));
            text(colCnt-.5,m-rowCnt+.5,numberString, ...
                 'HorizontalAlignment','center', ...
                 'Color',fontColor, ...
                 'FontWeight',fontWeight, ...
                 'FontSize',fontSize);
        end;
    end;

set(gca,'Box','on', ...
        'Visible','on', ...
        'xLim',[0 n], ...
        'xGrid','on', ...
        'xTickLabel',[], ...
        'xTick',0:n, ...
        'yGrid','on', ...
        'yLim',[0 m], ...
        'yTickLabel',[], ...
        'yTick',0:m, ...
        'DataAspectRatio',[1, 1, 1], ... 
        'GridLineStyle',':', ...
        'LineWidth',3, ...
        'XColor',gridColor, ...
        'YColor',gridColor);

title(matrixName,'FontSize',fontSize,'FontWeight',fontWeight);


function[G,AIC,MDL,halfmask] = polymodel_ic99(X,B,P,M)
%
%
%
%
X=X(:);
N=length(X);
t=(0:N-1)';
H=repmat(t,1,P+1).^repmat(0:P,N,1);
b_hat=inv(H'*H)*H'*X;

mask=fliplr(detobi(1:2^(P+1)-1));
msize=length(mask);
r_hat=X-H*b_hat;                    
%r_tilde=(r_hat-mean(r_hat))/sqrt(1-size(mask,2)/N);   
r_tilde=sqrt(N/M)*(r_hat-mean(r_hat))/sqrt(1-size(mask,2)/N);  
halfsize=ceil(msize/2);    % we do not have to look at the other half as we assume
 									% that the signal has a_0=0 
halfmask=mask(1:halfsize,:);
G=zeros(1,halfsize);  
AIC=zeros(1,halfsize); 
MDL=zeros(1,halfsize); 
for ii=1:halfsize,
  ind=find(mask(ii,:)~=0);
  Hm=H(:,ind);
  b_hat_m=inv(Hm'*Hm)*Hm'*X;
  X_hat=Hm*b_hat_m;
  r_hat=X-X_hat;
  r=r_hat-mean(r_hat);
  r_hat1=(r_hat-mean(r_hat))/sqrt(1-length(ind)/N);
  AIC(ii)=log(mean(r_hat1.^2))+2*(length(ind))/N;
  MDL(ii)=log(mean(r_hat1.^2))+log(N)*(length(ind))/N;
  SSE=zeros(1,B);
  for b=1:B,
    r_tilde_star=boot(N,r_tilde); 
    X_star=X_hat+r_tilde_star;
    Hmstar=Hm(1:N,:);
%    X_star=X_hat(1:M)+r_tilde_star(1:M);
%    Hmstar=Hm(1:M,:);
    bstarm=inv(Hmstar'*Hmstar)*Hmstar'*X_star;
    Xstar=Hm*bstarm; 
    SSE(b)=mean((X-Xstar).^2);    
  end;
  G(ii)=mean(SSE);
end;


function b = detobi(d, n, p)
%
%
%
d = d(:);
len_d = length(d);
if min(d) < 0
    error('Cannot convert a negative number');
end;

% assign the length
if nargin < 2;
    tmp = max(d);
    b1 = [];
    while tmp > 0
        b1 = [b1 rem(tmp, 2)];
        tmp = floor(tmp/2);
    end;
    n = length(b1);
end;
if nargin < 3
    p = 2;
end;

% initial value
b = zeros(len_d, n);

% parameter assignment
for i = 1 : len_d
    j = 1; 
    tmp = d(i);
    while (j <= n) & (tmp > 0)
        b(i, j) = rem(tmp, p);
        tmp = floor(tmp/p);
        j = j + 1;
    end;
end;

