function popupcb(popup)
%Callback function for popup menus

popuph=gcbo;
popupname=get(findobj(gcbf,'Tag',['StaticText_pop' num2str(popup)]),'String');
funlist=get(popuph,'UserData');
choice=get(popuph,'Value');
options=get(popuph,'String');

%Check if custom option has been selected
if strcmp(options{choice},'custom...')
  X=[1 2 3; 4 5 6]; [n,MC]=size(X); B=MC; %Test values
  switch popupname
  case 'STATISTIC'
    prompt={'Enter Matlab expression with X as placeholder for the sample:'};
    def={funlist{choice}}; %previous custom Matlab expression
    errmsg='Your Matlab expression must be able to work on each column if X is a matrix.  Please try again.';
    rightsize=[1 MC 1 n];
  case 'DISTRIBUTION'
    prompt={'Enter Matlab expression with n and MC as placeholders for number of samples and number of Monte Carlo runs respectively:'};
    def={funlist{choice}}; %previous custom Matlab expression
    errmsg='Your Matlab expression must return an n-by-MC matrix.  Please try again.';
    rightsize=[n MC MC n];
  case 'PARAMETRIC DIST.'
    prompt={'Enter Matlab expression with X, n and B as placeholders for the original sample, the number of samples and the number of bootstrap replications respectively:'};
    def={funlist{choice}}; %previous custom Matlab expression
    errmsg='Your Matlab expression must return an n-by-B matrix.  Please try again.';
    rightsize=[n B B n];
  case 'SIGNAL'
    prompt={'Enter Matlab expression with n as placeholder for the number of samples:'};
    def={funlist{choice}}; %previous custom Matlab expression
    errmsg='Your Matlab expression must return a column vector of size n.  Please try again.';
    rightsize=[n 1 MC 1];
  case 'NOISE'
    if choice<length(funlist) %need to get noise variance
      prompt={'Enter Matlab expression for the noise with n and MC as placeholders for number of samples and number of Monte Carlo runs respectively:';'Enter noise variance:'};
      def={funlist{choice};num2str(funlist{end})};
    else
      prompt={'Enter Matlab expression for the noise with n and MC as placeholders for number of samples and number of Monte Carlo runs respectively:'};
      def={funlist{choice}}; %previous custom Matlab expression
    end
    errmsg='Your Matlab expression must return an n-by-MC matrix.  Please try again.';
    rightsize=[n MC MC n];
  case {'COMMON SIGNAL','BACKGROUND NOISE','SENSOR NOISE'}
    if choice<length(funlist) %need to get noise variance
      prompt={'Enter Matlab expression with n and MC as placeholders for number of samples and number of Monte Carlo runs respectively:';'Enter time delay:'};
      def={funlist{choice};num2str(funlist{end})};
    else
      prompt={'Enter Matlab expression with n and MC as placeholders for number of samples and number of Monte Carlo runs respectively:'};
      def={funlist{choice}}; %previous custom Matlab expression
    end
    errmsg='Your Matlab expression must return an n-by-MC matrix.  Please try again.';
    rightsize=[n MC MC n];
  otherwise
    notyet;
    return;
  end

  %Get Matlab expression and save in user data of the popup menu
  while 1
    matlabexp=inputdlg(prompt,'',1,def);
    if isempty(matlabexp) %Cancel has been pressed
      break;
    end
    def=matlabexp;
    result=eval(matlabexp{1},'tryagain');
    if ~isempty(result)
      X=X'; MC=n; n=B; B=MC;
      res2=eval(matlabexp{1});
      X=X'; MC=n; n=B; B=MC; %reset the test data
      if any([size(result) size(res2)]~=rightsize)
        h=errordlg(errmsg);
        set(h,'WindowStyle','modal');
        uiwait(h);
      elseif length(matlabexp)>1 %there is more input to check
        v=str2num(matlabexp{2});
        if isempty(v) | v<=0 %input is not a positive number
          h=errordlg('Please enter a positive number for the second input.');
          set(h,'WindowStyle','modal');
          uiwait(h);
        else %Matlab expressions appear valid
          if strcmp(popupname,'COMMON SIGNAL') | strcmp(popupname,'BACKGROUND NOISE')
            v=ceil(v); %round up time delay to an integer
          end
          funlist{end-1}=matlabexp{1};
          funlist{end}=v;
          set(popuph,'UserData',funlist);
          break;
        end
      else %Matlab expression appears valid
        funlist{end}=matlabexp{1};
        set(popuph,'UserData',funlist);
        break;
      end
    end
  end
end

%Show selected option graphically if appropriate
switch popupname
case {'SIGNAL','NOISE'}
  n=str2num(get(findobj(gcbf,'Tag','EditText_N'),'String'));
  MC=1;
  matlabexp=funlist{choice};
  if strcmp(popupname,'NOISE') & length(options)<length(funlist)
    %display noise variance in title
    if choice==length(options) %noise variance was provided by user
      v=funlist{end};
    else
      v=1; %default variance
    end
    vtext=sprintf(' (\\sigma^2= %g)',v);
  else
    vtext='';
  end
  plot1(eval(matlabexp),[popupname vtext],'Sample','');
case {'COMMON SIGNAL','BACKGROUND NOISE','SENSOR NOISE'}
  if strcmp(popupname,'SENSOR NOISE')
    MC=2;
    d=0;
    dtext=' (blue: Sensor 1, green: Sensor 2)';
  else
    MC=1;
    if choice==length(options) %time delay was provided by user
      d=funlist{end};
    else
      if strcmp(popupname,'COMMON SIGNAL')
        d=5; %default time delay for common signal
      else
        d=10; %default time delay for background noise
      end
    end
    dtext=sprintf(' (blue: Sensor 1, green: Sensor 2, time delay= %d)',d);
  end
  n=str2num(get(findobj(gcbf,'Tag','EditText_N'),'String'))+d;
  x=eval(funlist{choice});
  if d>0 %need to plot delayed signal
    x=[x(1+d:end) x(1:end-d)];
  end
  plot1(x,[popupname dtext],'Sample','');
end

%For variance stabilising bootstrap:
%If the correlation coefficient statistic is selected, change the DISTRIBUTION
%popup menu to contain only bivariate distribution choices.
if strcmp(popupname,'STATISTIC')
  if strcmp(options{choice},'correlation coefficient')
    s=get(findobj(gcbf,'String','DISTRIBUTION','Visible','on'),'Tag');
    set(findobj(gcbf,'Tag',['PopupMenu' s(end)]), ...
      'String',{'Bi. Normal (rho=0.5)'}, ...
      'Value',1, ...
      'UserData',{'birandn(0.5,n,MC)'} ...
    );
    h=findobj(gcbf,'String','PARAMETRIC DIST.','Visible','on');
    if ~isempty(h) %need to change parametric distribution popup menu
      s=get(h,'Tag');
      set(findobj(gcbf,'Tag',['PopupMenu' s(end)]), ...
        'String',{'Bivariate Normal'}, ...
        'Value',1, ...
        'UserData',{'birandn(corr(X),n,B)'} ...
      );
    end
  else %provide univariate distributions
    s=get(findobj(gcbf,'String','DISTRIBUTION','Visible','on'),'Tag');
    h=findobj(gcbf,'Tag',['PopupMenu' s(end)]);
    if length(get(h,'String'))==1 %kludge to check if we need to update popup
      set(h, ...
        'String',{'N(10,25)';'N(0,1)';'U(0,1)';'custom...'}, ...
        'Value',1, ...
        'UserData',{'5*randn(n,MC)+10';'randn(n,MC)';'rand(n,MC)';'trnd(4,n,MC)'} ...
      );
    end
    h=findobj(gcbf,'String','PARAMETRIC DIST.','Visible','on');
    if ~isempty(h) %need to change parametric distribution popup menu
      if length(get(h,'String'))==1 %kludge to check if we need to update popup
        s=get(h,'Tag');
        set(findobj(gcbf,'Tag',['PopupMenu' s(end)]), ...
        'String', {'Normal';'Uniform';'custom...'}, ...
        'Value',1, ...
        'UserData',{'std(X)*randn(n,B)+mean(X)';'max(X)*rand(n,B)';'ceil(n*rand(n,B))'} ...
      );
      end
    end
  end
end

function flag=tryagain()
h=errordlg('You entered an invalid Matlab expression.  Please try again.');
set(h,'WindowStyle','modal');
uiwait(h);
flag=[];

function plot1(x,ttl,xlbl,ylbl)
outputh=findobj(gcbf,'Tag','Axes1');
fontsize=get(outputh,'FontSize');
axes(outputh); cla;
plot(x); title(ttl); xlabel(xlbl); ylabel(ylbl); grid on;
%Ensure crucial properties are not changed after plotting
set(outputh,'Tag','Axes1','FontUnits','normalized','FontSize',fontsize);
set(get(outputh,'title'),'FontUnits','normalized','FontSize',fontsize);
set(get(outputh,'xlabel'),'FontUnits','normalized','FontSize',fontsize);
set(get(outputh,'ylabel'),'FontUnits','normalized','FontSize',fontsize);
